#include <pybind11/pybind11.h>
#include <pybind11/stl.h>
#include "bert_tokenizer.h"

namespace py = pybind11;

// Define MACRO_STRINGIFY for version info
#define STRINGIFY(x) #x
#define MACRO_STRINGIFY(x) STRINGIFY(x)

PYBIND11_MODULE(_core, m) {
    m.doc() = R"pbdoc(
        FlashBertTokenizer Python bindings
        ---------------------------------

        .. currentmodule:: flash_tokenizer

        .. autosummary::
           :toctree: _generate

           FlashBertTokenizer
    )pbdoc";

    py::class_<FlashBertTokenizer>(m, "FlashBertTokenizer")
        .def(py::init<const std::string &, bool, int>(),
             py::arg("vocab_file"),
             py::arg("do_lower_case") = true,
             py::arg("max_input_chars_per_word") = 256)
        .def("tokenize", &FlashBertTokenizer::tokenize)
        .def("convert_tokens_to_ids", &FlashBertTokenizer::convert_tokens_to_ids)
        .def("convert_ids_to_tokens", &FlashBertTokenizer::convert_ids_to_tokens)
        .def("__call__", &FlashBertTokenizer::operator());

    #ifdef VERSION_INFO
        m.attr("__version__") = MACRO_STRINGIFY(VERSION_INFO);
    #else
        m.attr("__version__") = "dev";
    #endif
}
